@@ -48,6 +48,8 @@ my $ARCH_REGEX = Lintian::Data->new('binaries/arch-regex', qr/\s*\~\~/o,
     sub { return qr/$_[1]/ });
 my $ARCH_64BIT_EQUIVS
   = Lintian::Data->new('binaries/arch-64bit-equivs', qr/\s*\=\>\s*/);
+my $BINARY_SPELLING_EXCEPTIONS
+  = Lintian::Data->new('binaries/spelling-exceptions', qr/\s+/);
 
 my %PATH_DIRECTORIES = map { $_ => 1 } qw(
   bin/ sbin/ usr/bin/ usr/sbin/ usr/games/ );
@@ -190,7 +192,8 @@ sub run {
             # According to the binutils documentation[1], the profiling symbol
             # can be named "mcount", "_mcount" or even "__mcount".
             # [1] http://sourceware.org/binutils/docs/gprof/Implementation.html
-            if ($sec =~ /^GLIBC_.*/ and $sym =~ m{\A _?+ _?+ mcount \Z}xsm){
+            if (    $sec =~ /^GLIBC_.*/
+                and $sym =~ m{\A _?+ _?+ (gnu_)?+mcount(_nc)?+ \Z}xsm) {
                 $is_profiled = 1;
             } elsif ($arch ne 'hppa') {
                 # This code was used to detect profiled code in Wheezy
@@ -386,13 +389,7 @@ sub run {
         my $strings = slurp_entire_file($info->strings($file));
         my $exceptions = {
             %{ $group->info->spelling_exceptions },
-            'teH' => 1, # From #711207
-            'tEH' => 1, # From #782902
-            'tEh' => 1, # From #782902, too
-            'ang' => 1, # The Go stdlib html/ package contains "ang;"
-            'writeN' => 1, # The Go stdlib text/tabwriter pkg contains "writeN"
-            'writeN' => 1, # The Go stdlib text/tabwriter pkg contains "writeN"
-            'ot' => 1, # The Go stdlib runtime/ package contains "ot"
+            map { $_ => 1} $BINARY_SPELLING_EXCEPTIONS->all
         };
         my $tag_emitter
           = spelling_tag_emitter('spelling-error-in-binary', $file);
@@ -517,7 +514,7 @@ sub run {
 
         # statically linked?
         if (!exists($objdump->{NEEDED})) {
-            if ($fileinfo =~ m/shared object/o) {
+            if ($fileinfo =~ m/(shared object|pie executable)/o) {
                 # Some exceptions: kernel modules, syslinux modules, detached
                 # debugging information and the dynamic loader (which itself
                 # has no dependencies).
@@ -559,7 +556,7 @@ sub run {
             my $is_shared = 0;
             my @needed;
             $needs_depends_line = 1;
-            $is_shared = 1 if index($fileinfo, 'shared object') != -1;
+            $is_shared = 1 if $fileinfo =~ m/(shared object|pie executable)/;
             @needed = @{$objdump->{NEEDED}} if exists($objdump->{NEEDED});
             for my $lib (@needed) {
                 if ($lib =~ /^libc\.so\.(\d+.*)/) {
