@@ -23,6 +23,8 @@ use strict;
 use warnings;
 use autodie;
 
+use List::MoreUtils qw(none);
+
 use Lintian::Tags qw(tag);
 use Lintian::Check qw(check_maintainer);
 use Lintian::Data;
@@ -30,6 +32,8 @@ use Lintian::Util qw(get_file_checksum);
 
 my $KNOWN_DISTS = Lintian::Data->new('changes-file/known-dists');
 my $SIGNING_KEY_FILENAMES = Lintian::Data->new('common/signing-key-filenames');
+my $DERIVATIVE_VERSIONS= Lintian::Data->new('changes-file/derivative-versions',
+    qr/\s*~~\s*/, sub { $_[1]; });
 
 sub run {
     my (undef, undef, $info, undef, $group) = @_;
@@ -198,11 +202,11 @@ sub run {
         # Ensure all orig tarballs have a signature if we have an upstream
         # signature.
         if (   $has_signing_key
-            && $file =~ m/\.orig(-[A-Za-z\d-]+)?\.tar\./
+            && $file =~ m/(^.*\.orig(?:-[A-Za-z\d-]+)?\.tar)\./
             && $file !~ m/\.asc$/
             && !$info->repacked) {
             tag 'orig-tarball-missing-upstream-signature', $file
-              unless exists $files->{"$file.asc"};
+              if none { exists $files->{"$_.asc"} } ($file, $1);
         }
 
         # check section
@@ -252,6 +256,14 @@ sub run {
           if $seen != $expected;
     }
 
+    # Check version field
+    my $version = $info->field('version');
+    foreach my $re ($DERIVATIVE_VERSIONS->all) {
+        next if $version =~ m/$re/;
+        my $explanation = $DERIVATIVE_VERSIONS->value($re);
+        tag 'invalid-version-number-for-derivative', $version,"($explanation)";
+    }
+
     return;
 }
 
