@@ -330,8 +330,7 @@ my @file_checks = (
 # to the debian/ source directory
 our @EOL_TERMINATORS_FILES = qw(control changelog);
 
-# List of files to check for a trailing whitespace characters relative
-# to the debian/ source directory
+# List of files to check for a trailing whitespace characters
 our @TRAILING_WHITESPACE_FILES = (
     ['debian/changelog'        => qr,\s+\n$,],
     ['debian/control'          => qr,\s+\n$,],
@@ -398,11 +397,20 @@ sub run {
         my $path = $info->index_resolved_path($file->[0]);
         next if not $path or not $path->is_open_ok;
         my $fd = $path->open;
+        my @empty_lines;
         while (my $line = <$fd>) {
+            if ($line eq "\n") {
+                push @empty_lines, $.;
+                next;
+            }
+            @empty_lines = (); # reset; line is not empty
             tag 'file-contains-trailing-whitespace', "$path (line $.)"
               if ($line =~ $file->[1]);
         }
         close($fd);
+        for my $num (@empty_lines) {
+            tag 'file-contains-trailing-whitespace', "$path (line $num)";
+        }
     }
 
     if (my $pycompat = $info->index_resolved_path('debian/pycompat')) {
@@ -439,6 +447,12 @@ sub run {
         }
     }
 
+    foreach my $file ($info->sorted_orig_index) {
+        tag 'source-contains-empty-directory', $file
+          if $file->is_dir
+          and scalar($file->children) == 0;
+    }
+
     return;
 }    # </run>
 
@@ -1363,6 +1377,9 @@ sub _clean_block {
     # po comment may include html tag
     $text =~ s/\"\s?\v\#~\s?\"//gxms;
 
+    # strip .rtf paragraph marks (#892967)
+    $text =~ s/\\par\b//gxms;
+
     $text =~ s/\\url[{][^}]*?[}]/ /gxms;      # (la)?tex url
     $text =~ s/\\emph[{]/ /gxms;              # (la)?tex emph
     $text =~ s/\\href[{][^}]*?[}]
@@ -1572,9 +1589,12 @@ sub _ships_examples {
     return 1 if not @procs;
     foreach my $binpkg (@procs) {
         my $name = $binpkg->pkg_name;
+        # If we have an -examples package, assume we ship examples.
         return 1 if $name =~ m{-examples$};
         my @files = $binpkg->info->sorted_index;
-        return 1 if any { m{^usr/share/doc/$name/examples/$} } @files;
+        # Check each package for a directory (or symlink) called "examples".
+        return 1
+          if any { m{^usr/share/doc/(.+/)?examples/?$} } @files;
     }
     return;
 }
