@@ -54,6 +54,8 @@ our $KNOWN_UDEB_FIELDS = Lintian::Data->new('fields/udeb-fields');
 our $KNOWN_BUILD_PROFILES = Lintian::Data->new('fields/build-profiles');
 our $KNOWN_VCS_BROWSERS
   = Lintian::Data->new('fields/vcs-browsers', qr/\s*~~\s*/, sub { $_[1]; });
+my $KNOWN_INSECURE_HOMEPAGE_URIS
+  = Lintian::Data->new('fields/insecure-homepage-uris');
 
 our %KNOWN_ARCHIVE_PARTS = map { $_ => 1 } ('non-free', 'contrib');
 
@@ -240,7 +242,6 @@ our $OBSOLETE_PACKAGES
   = Lintian::Data->new('fields/obsolete-packages',qr/\s*=>\s*/);
 our $VIRTUAL_PACKAGES   = Lintian::Data->new('fields/virtual-packages');
 our $SOURCE_FIELDS      = Lintian::Data->new('common/source-fields');
-our $MAIL_TRANSPORT_AGENTS= Lintian::Data->new('fields/mail-transport-agents');
 
 sub run {
     my ($pkg, $type, $info, $proc, $group) = @_;
@@ -675,11 +676,10 @@ sub run {
         if ($homepage=~ m,bioconductor\.org/packages/.*/bioc/html/.*\.html*$,){
             tag 'homepage-for-bioconductor-package-not-canonical', $orig;
         }
-        if (   $homepage =~ m,^ftp://,
-            or $homepage
-            =~m,^http://(?:[^\.]+\.)?(?:github\.com|metacpan\.org|debian\.org|cran\.r-project\.org|bioconductor\.org)/,
-          ){
+        foreach my $re ($KNOWN_INSECURE_HOMEPAGE_URIS->all) {
+            next unless $homepage =~ m/$re/;
             tag 'homepage-field-uses-insecure-uri', $orig;
+            last;
         }
     } elsif (not $info->native) {
         if ($type eq 'source') {
@@ -792,11 +792,6 @@ sub run {
                   if ( &$is_dep_field($field)
                     && $known_obsolete_emacs{$alternatives[0][0]});
 
-                tag 'depends-on-mail-transport-agent-without-alternatives',
-                  $dep
-                  if &$is_dep_field($field)
-                  and $MAIL_TRANSPORT_AGENTS->known($dep);
-
                 for my $part_d (@alternatives) {
                     my ($d_pkg, undef, $d_version, undef, undef, $rest,
                         $part_d_orig)
@@ -925,6 +920,7 @@ sub run {
                     tag 'binary-package-depends-on-toolchain-package',
                       "$field: $part_d_orig"
                       if $KNOWN_TOOLCHAIN->known($d_pkg)
+                      and $is_dep_field
                       and not $pkg =~ m/^dh-/
                       and not $pkg =~ m/-(source|src)$/
                       and not $DH_ADDONS_VALUES{$pkg};
@@ -1294,47 +1290,6 @@ sub run {
           unless $bugs =~ m,\.debian\.org, or $pkg =~ /[-]dbgsym$/;
     }
 
-    #----- Python-Version
-
-    if (defined(my $pyversion = $info->field('python-version'))) {
-        unfold('python-version', \$pyversion);
-
-        my @valid = (
-            ['\d+\.\d+', '\d+\.\d+'],['\d+\.\d+'],
-            ['\>=\s*\d+\.\d+', '\<\<\s*\d+\.\d+'],['\>=\s*\d+\.\d+'],
-            ['current', '\>=\s*\d+\.\d+'],['current'],
-            ['all']);
-
-        my @pyversion = split(/\s*,\s*/, $pyversion);
-
-        if ($pyversion =~ m/^current/) {
-            tag 'python-version-current-is-deprecated';
-        }
-
-        if (@pyversion > 2) {
-            if (any { !/^\d+\.\d+$/ } @pyversion) {
-                tag 'malformed-python-version', $pyversion;
-            }
-        } else {
-            my $okay = 0;
-            for my $rule (@valid) {
-                if (
-                    $pyversion[0] =~ /^$rule->[0]$/
-                    && ((
-                               $pyversion[1]
-                            && $rule->[1]
-                            && $pyversion[1] =~ /^$rule->[1]$/
-                        )
-                        || (!$pyversion[1] && !$rule->[1]))
-                  ) {
-                    $okay = 1;
-                    last;
-                }
-            }
-            tag 'malformed-python-version', $pyversion unless $okay;
-        }
-    }
-
     #----- Dm-Upload-Allowed
 
     if (defined(my $dmupload = $info->field('dm-upload-allowed'))) {
@@ -1393,9 +1348,9 @@ sub run {
             } else {
                 $seen_vcs{$vcs}++;
             }
-            if ($uri =~ m{//([^.]+)\.debian\.org/}) {
+            if ($uri =~ m{//(.+)\.debian\.org/}) {
                 tag 'vcs-deprecated-in-debian-infrastructure', "vcs-$vcs", $uri
-                  if $1 ne 'salsa';
+                  unless $1 =~ m{^(?:salsa|git\.dgit)$};
             } else {
                 tag 'orphaned-package-not-maintained-in-debian-infrastructure',
                   "vcs-$vcs", $uri
@@ -1524,15 +1479,17 @@ sub unfold {
 
 sub check_field {
     my ($info, $field, $data) = @_;
-    my @seen;
-    $info->relation($field)->visit(sub { push @seen, $_; }, VISIT_PRED_NAME);
 
-    my $has_default_mta = any { $_ eq 'default-mta' } @seen;
-    my $has_mail_transport_agent = any { $_ eq 'mail-transport-agent' } @seen;
+    my $has_default_mta
+      = $info->relation($field)->matches(qr/^default-mta$/, VISIT_PRED_NAME);
+    my $has_mail_transport_agent = $info->relation($field)
+      ->matches(qr/^mail-transport-agent$/, VISIT_PRED_NAME);
+
+    tag 'default-mta-dependency-not-listed-first',"$field: $data"
+      if $info->relation($field)
+      ->matches(qr/\|\s+default-mta/, VISIT_OR_CLAUSE_FULL);
 
     if ($has_default_mta) {
-        tag 'default-mta-dependency-not-listed-first',"$field: $data"
-          if $seen[0] ne 'default-mta';
         tag 'default-mta-dependency-does-not-specify-mail-transport-agent',
           "$field: $data"
           unless $has_mail_transport_agent;
