@@ -56,7 +56,7 @@ my (
     $RESOURCE_MANAGER, $LINTIAN_VERSION, $timestamp,
     $TEMPLATE_CONFIG_VARS,$HARNESS_STATE_DIR, $HISTORY_DIR,
     $HISTORY, $GRAPHS, $LINTIAN_ROOT,
-    $HTML_TMP_DIR,
+    $HTML_TMP_DIR, $SCOUR_ENABLED,
 );
 # FIXME: Should become obsolete if gnuplot is replaced by R like piuparts.d.o /
 # reproducible.d.n is using
@@ -120,7 +120,7 @@ my (%statistics, %tag_statistics);
 my (%by_maint, %by_uploader, %by_tag, %maintainer_table, %delta);
 my @attrs = qw(maintainers source-packages binary-packages udeb-packages
   errors warnings info experimental pedantic overridden groups-known
-  groups-backlog classifications);
+  groups-backlog classifications groups-with-errors);
 
 sub required_cfg_value {
     my (@keys) = @_;
@@ -208,6 +208,16 @@ sub init_globals {
             $GRAPHS = 0;
             print "No graphs as \"gnuplot\" is not in PATH\n";
         }
+        if ($GRAPHS) {
+            if (check_path('scour')) {
+                $SCOUR_ENABLED = 1;
+                print "Minimizing generated SVG files (scour is in PATH)\n";
+            } else {
+                $SCOUR_ENABLED = 0;
+                print 'No minimization of generated SVG files'
+                  . " as \"scour\" is not in PATH\n";
+            }
+        }
     } else {
         $HISTORY = 0;
         $GRAPHS = 0;
@@ -223,7 +233,7 @@ sub init_globals {
     # Provide a default URL for the source code.  It might not be correct for
     # the given installation, but it is better than nothing.
     $TEMPLATE_CONFIG_VARS->{'LINTIAN_SOURCE'}
-      //= 'https://anonscm.debian.org/git/lintian/lintian.git';
+      //= 'https://salsa.debian.org/lintian/lintian.git';
 
     my $profile = dplint::load_profile();
 
@@ -404,17 +414,40 @@ sub process_data {
         my $description = $info->description('html', '    ');
         my ($count, $overrides) = (0, 0);
         my $tmpl = 'tag-not-seen';
+        my $shown_count = 0;
+        my $tag_list = $by_tag{$tag};
+        my $tag_limit_total = 1024;
+        my $tag_limit_per_package = 3;
+
         if (exists $by_tag{$tag}) {
             $tmpl = 'tag';
             $count = $tag_statistics{$tag}{'count'};
             $overrides = $tag_statistics{$tag}{'overrides'};
+            $shown_count = $count + $overrides;
+        }
+        if ($shown_count > $tag_limit_total) {
+            my (@replacement_list, %seen);
+            for my $orig_info (
+                sort { $a->{pkg_info}{package} cmp $b->{pkg_info}{package} }
+                @{$tag_list}) {
+                my $pkg_info = $orig_info->{pkg_info};
+                my $key
+                  = "$pkg_info->{package} $pkg_info->{type} $pkg_info->{version}";
+                next if ++$seen{$key} > $tag_limit_per_package;
+                push(@replacement_list, $orig_info);
+                last if @replacement_list >= $tag_limit_total;
+            }
+            $tag_list = \@replacement_list;
+            $shown_count = scalar(@replacement_list);
         }
 
         my %maint_data = (
             description => $description,
             tag         => $tag,
             code        => $info->code,
-            tags        => $by_tag{$tag},
+            tags        => $tag_list,
+            shown_count => $shown_count,
+            tag_limit_per_package => $tag_limit_per_package,
             graphs      => $GRAPHS,
             graphs_days => $GRAPHS_RANGE_DAYS,
             statistics  => {
@@ -464,6 +497,8 @@ sub process_data {
         my $trace_file= "${path}/project/trace/${trace_basename}";
         my $mirror_timestamp = slurp_entire_file($trace_file);
         $mirror_timestamp =~ s/\n.*//s;
+        $mirror_timestamp
+          = safe_qx('date', '-u', '--rfc-822', '-d', $mirror_timestamp);
         my %info = (
             'name' => $archive,
             'architectures' => $architectures,
@@ -508,16 +543,8 @@ sub setup_output_dir {
             'in'  => $lintian_log_file,
             'out' => "$output_dir/lintian.log.gz",
         );
-        $RESOURCE_MANAGER->install_resource(
-            'lintian.log',
-            {
-                'install_method' => 'copy',
-                'source_file' => $lintian_log_file,
-            });
         run_cmd(\%opts, 'gzip', '-9nc');
         $RESOURCE_MANAGER->install_resource("$output_dir/lintian.log.gz");
-        symlink($RESOURCE_MANAGER->resource_URL('lintian.log'),
-            "$output_dir/lintian.log");
         symlink($RESOURCE_MANAGER->resource_URL('lintian.log.gz'),
             "$output_dir/lintian.log.gz");
     }
@@ -591,6 +618,13 @@ sub collect_statistics {
     $statistics{'groups-known'} = scalar(keys(%{$state_cache->{'groups'}}));
     $statistics{'groups-backlog'}
       = scalar(find_backlog($LINTIAN_VERSION,$state_cache));
+    my $pkgs_w_errors = 0;
+    for my $group_data (values(%{$state_cache->{'groups'}})) {
+        $pkgs_w_errors++
+          if exists($group_data->{'processing-errors'})
+          and $group_data->{'processing-errors'};
+    }
+    $statistics{'groups-with-errors'} = $pkgs_w_errors;
 
     for my $attr (@attrs) {
         my $old = $old_statistics->{$attr} || 0;
@@ -641,6 +675,14 @@ sub generate_package_index_packages {
     return;
 }
 
+sub run_scour {
+    my ($input_file, $output_file) = @_;
+    run_cmd('scour', '-i',$input_file, '-o',$output_file, '-q',
+        '--enable-id-stripping', '--enable-comment-stripping',
+        '--shorten-ids', '--indent=none');
+    return 1;
+}
+
 sub update_history_and_make_graphs {
     my ($attrs_ref, $statistics_ref, $tag_statistics_ref) = @_;
     # Update history.
@@ -701,9 +743,9 @@ sub update_history_and_make_graphs {
                 (my $label = $v) =~ s/\-[\w\d]+$//;
                 print {$common} "set label '$label' at '$versions{$v}',graph ",
                   "1.04 rotate by 90 font ',8'\n";
-            }
 
-            $last_version = $versions{$v};
+                $last_version = $versions{$v};
+            }
         }
         close($common);
 
@@ -711,6 +753,19 @@ sub update_history_and_make_graphs {
         run_cmd({ 'chdir' => $graph_dir},
             'gnuplot',"$LINTIAN_ROOT/reporting/graphs/statistics.gpi");
 
+        if ($SCOUR_ENABLED) {
+            # Do a little "rename" dance to ensure that we keep the
+            # "statistics.svg"-basename without having to use a
+            # subdirectory.
+            rename(
+                "${graph_dir}/statistics.svg",
+                "${graph_dir}/_statistics-orig.svg"
+            );
+            run_scour(
+                "${graph_dir}/_statistics-orig.svg",
+                "${graph_dir}/statistics.svg"
+            );
+        }
         $RESOURCE_MANAGER->install_resource("${graph_dir}/statistics.svg");
     }
 
@@ -735,11 +790,23 @@ sub update_history_and_make_graphs {
     }
 
     if ($GRAPHS) {
+        my $svg_dir = "${graph_dir}/tags";
         close($gnuplot_fd);
         run_cmd({'chdir' => $graph_dir}, 'gnuplot', 'call.gpi');
         unlink($commonf);
+        if ($SCOUR_ENABLED) {
+            # Obvious optimization potential; run scour in parallel
+            my $optimized_dir = "${graph_dir}/tags-optimized";
+            mkdir($optimized_dir);
+            print "Minimizing tag graphs; this may take a while ...\n";
+            for my $tag (sort(keys(%{$tag_statistics_ref}))) {
+                run_scour("${svg_dir}/${tag}.svg",
+                    "${optimized_dir}/${tag}.svg");
+            }
+            $svg_dir = $optimized_dir;
+        }
         for my $tag (sort(keys(%{$tag_statistics_ref}))) {
-            my $graph_file = "${graph_dir}/tags/${tag}.svg";
+            my $graph_file = "${svg_dir}/${tag}.svg";
             $RESOURCE_MANAGER->install_resource($graph_file);
         }
         delete_dir($graph_dir);
@@ -820,7 +887,7 @@ sub parse_lintian_log {
     # The "last_*" are optimizations to avoid computing the same
     # things over and over again when a package have multiple tags.
     my (%seen, $last_info, $last_maintainer, %unknown_member_id, $info,
-        $last_pi);
+        $last_pi, %map_maint);
     my %expanded_code = (
         E => 'errors',
         W => 'warnings',
@@ -842,7 +909,7 @@ sub parse_lintian_log {
         # Ignore unknown tags - happens if we removed a tag that is
         # still present in the log file.
         my $tag_info = $profile->get_tag($tag, 1);
-        next unless $tag_info;
+        next if not $tag_info or $tag_info->severity eq 'classification';
 
         # Update statistics.
         my $key = $expanded_code{$code};
@@ -953,7 +1020,8 @@ sub parse_lintian_log {
             # and, if so, map them to the same person as the previous
             # one.
 
-            $last_maintainer = $maintainer = map_maintainer($maintainer);
+            $last_maintainer = $maintainer
+              = map_maintainer(\%map_maint, $maintainer);
 
             # Update maintainer statistics.
             $statistics{maintainers}++ unless defined $by_maint{$maintainer};
@@ -995,7 +1063,7 @@ sub parse_lintian_log {
                             # d/control for some uploaders.
                             $uploader =~ s/^"(.*)" <(.*)>$/$1 <$2>/;
                         }
-                        $uploader = map_maintainer($uploader);
+                        $uploader = map_maintainer(\%map_maint, $uploader);
                         next if $uploader eq $maintainer;
                         $by_uploader{$uploader}{$source}{$source_version}
                           = $list_ref;
@@ -1015,24 +1083,15 @@ sub parse_lintian_log {
 # Deduplicate maintainers.  Maintains a cache of the maintainers we've seen
 # with a given e-mail address and returns the maintainer string that we
 # should use (which is whatever maintainer we saw first with that e-mail).
-
-# PerlCritic apparently does not recognise this way of "encapsulating"
-# variables.  Though, these could probably be replaced by "state"
-# variables.
-## no critic (ControlStructures::ProhibitUnreachableCode)
-{
-    my (%urlmap);
-
-    sub map_maintainer {
-        my ($maintainer) = @_;
-        my $url = maintainer_url($maintainer);
-        if ($urlmap{$url} && $urlmap{$url} ne $maintainer) {
-            $maintainer = $urlmap{$url};
-        } else {
-            $urlmap{$url} = $maintainer;
-        }
-        return $maintainer;
+sub map_maintainer {
+    my ($urlmap, $maintainer) = @_;
+    my $url = maintainer_url($maintainer);
+    if (defined(my $res = $urlmap->{$url})) {
+        $maintainer = $res;
+    } else {
+        $urlmap->{$url} = $maintainer;
     }
+    return $maintainer;
 }
 
 # Quote special characters for HTML output.
@@ -1091,8 +1150,8 @@ sub output_template {
                 %{$data},
             }) or die "Filling out footer of $file: $Text::Template::ERROR\n";
     };
-    $data->{config_vars} ||= $TEMPLATE_CONFIG_VARS,
-      open(my $fd, '>:encoding(UTF-8)', "$HTML_TMP_DIR/$file");
+    $data->{config_vars} ||= $TEMPLATE_CONFIG_VARS;
+    open(my $fd, '>:encoding(UTF-8)', "$HTML_TMP_DIR/$file");
     $template->fill_in(OUTPUT => $fd, HASH => $data)
       or die "filling out $file failed: $Text::Template::ERROR\n";
     close($fd);
