@@ -307,7 +307,7 @@ sub index {
         'index_file' => 'index',
         'index_owner_file' => 'index-owner-id',
         'fs_root_sub' => 'unpacked',
-        'has_anchored_root_dir' => 1,
+        'has_anchored_root_dir' => 0,
         'file_info_sub' => 'file_info',
     };
     return $self->_fetch_index_data($load_info, $file);
@@ -426,7 +426,7 @@ sub _fetch_index_data {
         '_collect' => $self,
         '_collect_path_sub' => $load_info->{'fs_root_sub'},
         '_collect_file_info_sub' => $load_info->{'file_info_sub'},
-        'has_anchored_root_dir' => $load_info->{'as_anchored_root_dir'},
+        'has_anchored_root_dir' => $load_info->{'has_anchored_root_dir'},
     );
 
     if ($indexown) {
@@ -445,6 +445,8 @@ sub _fetch_index_data {
         # "fake" dates.
         ($perm,$ownership,$size,$date,$time,$name)
           = $line=~ /^(.{10}) (.*?) (\d+) ([-\d]{10}) ([:\d]{5,8}) (.*)$/;
+        croak "cannot parse tar output from $index: $line"
+          unless defined $perm;
         $ownership =~ s/\s+$//;
 
         $file{'date_time'} = "${date} ${time}";
@@ -507,6 +509,10 @@ sub _fetch_index_data {
         } elsif ($raw_type eq 'l') {
             ($name, $file{link}) = split ' -> ', $name, 2;
             $file{link} = dequote_name($file{link}, 0);
+        } elsif ($raw_type eq 'd') {
+            # Ensure directory names always end with  / or we will add them
+            # multiple times to our index.
+            $name .= '/' if substr($name, -1) ne '/';
         }
         # We store the name here, but will replace it later.  The
         # reason for storing it now is that we may need it during the
